#!/usr/bin/env python
"""
    Library functions for cij_testcases
"""
import os
import datetime
import ansi2html
import jinja2
import cij.reporter
import cij

EXTS = {
    "TESTPLANS": [".plan"],
    "TESTSUITES": [".suite"],
    "TESTCASES": [".py", ".sh"],
}


def index(search_path, ext=None):
    """@returns a set of testcases in the given search_path"""

    if ext is None:
        ext = "TCASE"

    tcases = set([])
    for _, _, files in os.walk(search_path):
        for tc_fname in files:
            if os.path.splitext(tc_fname)[-1] in EXTS[ext]:
                tcases.add(tc_fname)

    return sorted(list(tcases))


def construct_dset(evars):
    """
    Create a dataset = { "group_names": [ testcase, .. , testcase ] }
    """

    fnames = index(evars["TESTCASES"], "TESTCASES")

    dset = {
        "group_names": [],
    }

    for fname in fnames:
        group = fname.split("_")[0]
        if group not in dset:
            dset[group] = []
            dset["group_names"].append(group)

        fpath = os.sep.join([evars["TESTCASES"], fname])

        tcase = {
            "fpath": fpath,
            "name": fname,
            "fname": fname,
            "descr": "",
            "descr_long": "",
            "src_content": open(fpath, "r").read()
        }

        descr, descr_long = cij.reporter.tcase_parse_descr(tcase)

        tcase["descr"] = descr
        tcase["descr_long"] = descr_long

        dset[group].append(tcase)

    return dset


def dset_to_html(dset, tmpl_fpath):
    """
    @returns A HTML representation of the given 'trun' using the template at
    'tmpl_fpath'
    """

    def stamp_to_datetime(stamp):
        """Create a date object from timestamp"""

        return datetime.datetime.fromtimestamp(int(stamp))

    def strftime(dtime, fmt):
        """Create a date object from timestamp"""

        return dtime.strftime(fmt)

    def ansi_to_html(ansi):
        """Convert the given ANSI text to HTML"""

        conv = ansi2html.Ansi2HTMLConverter(
            scheme="solarized",
            inline=True
        )
        html = conv.convert(ansi, full=False)

        with open("/tmp/jazz.html", "w") as html_file:
            html_file.write(html)

        return html

    tmpl_dpath = os.path.dirname(tmpl_fpath)
    tmpl_fname = os.path.basename(tmpl_fpath)

    env = jinja2.Environment(
        autoescape=True,
        loader=jinja2.FileSystemLoader(tmpl_dpath)
    )
    env.filters['stamp_to_datetime'] = stamp_to_datetime
    env.filters['strftime'] = strftime
    env.filters['ansi_to_html'] = ansi_to_html

    tmpl = env.get_template(tmpl_fname)

    return tmpl.render(dset=dset)


def main(args, evars):
    """
    Search for testcases, parse their descriptions and write them to report
    """

    html_fpath = os.sep.join([args.output, "testcases.html"])

    cij.emph("html_fpath: %r" % html_fpath)

    dset = construct_dset(evars)

    with open(html_fpath, 'w') as html_fd:
        html = dset_to_html(dset, args.tmpl_fpath)
        html_fd.write(html)

    return 0



"""
    Utilities for writing Python tests with CIJOE
"""
import sys
import os
import yaml
import cij.board
import cij.block
import cij.nvme
import cij.lnvm
import cij.util
import cij.nvm
import cij.ssh
import cij


PASS = 0
FAIL = 1
SKIP = 2
REQS = []


def tindex(spath=None):
    """
    Lists tindex in CIJ_TESTCASES

    @Returns On success, a list of filenames is returned. On error, None is
    returned
    """

    spath = spath if spath else os.environ.get("CIJ_TESTCASES", None)
    if spath is None:
        return None

    tests = []                          # Look for .sh files
    for root, _, files in os.walk(spath):
        if root != spath:
            continue

        tests += [f for f in files if f[-3:] in [".sh", ".py"]]

    return tests


def envs():
    """
    Return variables defined by modules required by test
    """

    variables = {}

    for req in REQS:
        prefix = req.upper()
        variables[prefix] = cij.env_to_dict(
            prefix, getattr(cij, req).REQUIRED + getattr(cij, req).EXPORTED
        )

    return variables


def require(req):
    """Add test requirement"""

    REQS.append(req)


def enter():
    """Enter the test, check requirements and setup aux. environment"""

    if cij.ssh.env():
        tfail("cij.test: invalid SSH environment")

    for req in REQS:
        if getattr(cij, req).env():
            tfail()

    cij.emph("cij.test: entering test")


def texit(msg=None, rcode=1):
    """Exit the test"""

    msg = ", msg: %r" % msg if msg else ""

    if rcode:
        cij.err("cij.test: FAILED%s" % msg)
    else:
        cij.good("cij.test: PASSED%s" % msg)

    sys.exit(rcode)


def tpass(msg=None):
    """Testing: Exit test indicating test passed"""

    texit(msg, 0)


def tfail(msg=None):
    """Testing: Exit test indicating test failed"""

    texit(msg, 1)


def command(cmd, ssh=True, shell=True, echo=True):
    """
    Execute the given 'cmd'

    @returns (rcode, stdout, stderr)
    """

    if ssh:
        return cij.ssh.command(cmd, shell, echo)
    else:
        return cij.util.execute(cmd, shell, echo)


def command_to_struct(cmd):
    """
    Same as `command` except it tries to convert stdout to struct

    @returns (rcode, struct, stderr, struct)
    """

    struct = None

    rcode, stdout, stderr = command(cmd)

    try:
        lines = stdout.splitlines()

        for line_number, line in enumerate(lines):
            if line.strip().startswith("#"):
                break
        struct = yaml.load("\n".join(lines[line_number:]))
    except Exception as exc:
        cij.err("could not parse stdout as yaml, exc: %r" % exc)

    return rcode, stdout, stderr, struct



#!/usr/bin/env python
"""
 cij_runner - Turns sh-scripts into `unittest` compatible tests

 Traditional shell-based testing would only provide a single pass/fail
 for all tests, by enveloping them in Python and unittest framework then
 the test-runner can make result output available as junit/xunit,
 and individual tests can expose pass/fail instead of a single pass/fail
 for all tests.

 It uses the following environment variables:

 CIJ_SUITE  - Test suite
 CIJ_ENV    - Test environment

In addition to having a correctly setup CIJOE installation, specifically:

 CIJ_ENVS       - Path to environments definitions
 CIJ_TESTCASES      - This is what it is all about, scripts here are wrapped.
 CIJ_MODULES    - To locate 'cijoe.sh'

Usage:

 # Run test-suite "enum", in environment "localhost", produce XML output and
 # shorten error output
 export CIJ_TEST_RES_ROOT=/tmp
 export CIJ_SUITE="enum"
 export CIJ_ENV="localhost"
 py.test $CIJ_ROOT/bin/cij_runner.py \
    --junitxml /tmp/test_results.xml \
    -r f \
    --tb=line

A bunch of other `unittest` compatible frameworks can be used, py.test, however,
seems to provide both decent XML and console output.

TODO: log output should be configurable/parameterizable
"""
from __future__ import print_function
from subprocess import Popen, STDOUT
import unittest
import os
from cij_tests import tindex

CIJ_ENV = os.environ.get("CIJ_ENV")
CIJ_SUITE = os.environ.get("CIJ_SUITE")

CIJ_ROOT = os.environ.get("CIJ_ROOT")
CIJ_ENVS = os.environ.get("CIJ_ENVS")
CIJ_TESTCASES = os.environ.get("CIJ_TESTCASES")
CIJ_MODULES = os.environ.get("CIJ_MODULES")

# We inherit the large amount of public methods from unittest.TestCase,
# nothing we can do about that so we disable pylint checking.
# pylint: disable=r0904
class MetaTest(unittest.TestCase):
    """
    This is a meta-class used as the basis for dynamically
    creating unittest.TestCase classes based on .sh files.
    """

    def meta_test(self):
        """
        meta_test: executes sh-scripts, pipes out+err to file
        """

        # Make sure we have 'cijoe.sh' to setup default environment
        cijoe_path = os.sep.join([CIJ_MODULES, "cijoe.sh"])
        if not os.path.exists(cijoe_path):
            Exception("Cannot find cijoe(%s)" % cijoe_path)

        # Make sure we have an environment defined
        env_path = os.sep.join([CIJ_ENVS, "%s.sh" % CIJ_ENV])
        if not os.path.exists(env_path):
            raise Exception("Cannot find env(%s)" % env_path)

        # Use naming conventions of Class and Method to obtain path
        _, cname, mname = self.id().split(".")
        if not cname.startswith("Test"):
            raise Exception("Invalid cname(%s)" % cname)
        if not mname.startswith("test_"):
            raise Exception("Invalid mname(%s)" % mname)

        suite_name = cname[len("Test"):]

        test_name = "_".join(mname.split("_")[1:-1])
        test_lang = mname.split("_")[-1]
        test_ext = ".sh" if test_lang == "SHELL" else ".py"

        test_path = os.sep.join([CIJ_TESTCASES, "%s%s" % (test_name, test_ext)])
        assert os.path.exists(test_path), \
            "The script for %s/%s does not exist at test_path: %s" % (
                suite_name, test_name, test_path)

        # Run the actual test-script and pipe stdout+stderr to log
        log_path = '/tmp/test-%s-%s-%s.log' % (suite_name, test_name, CIJ_ENV)

        with open(log_path, 'a') as log_fd:

            if test_lang == "PYTHON":
                cmd = [
                    'bash', '-c',
                    'source %s && '
                    'source %s && '
                    'python %s' % (cijoe_path, env_path, test_path)
                ]
            else:
                cmd = [
                    'bash', '-c',
                    'source %s && '
                    'source %s && '
                    'source %s' % (cijoe_path, env_path, test_path)
                ]
            process = Popen(cmd, stdout=log_fd, stderr=STDOUT, cwd=CIJ_MODULES)
            process.wait()

            assert process.returncode == 0, "Ran: %s/%s, rcode: %d, see: %s" % (
                suite_name, test_name, process.returncode, log_path
            )

def cls_factory(cname):
    """
    Constructs a class named `cname` inheriting from `MetaTest`.
    """

    class Foo(MetaTest):
        """cls_factory product"""
        pass
    Foo.__name__ = cname

    return Foo

def produce_classes(search_path, suite_name):
    """Construct a class with a name picked up by test-collection"""

    classes = {}

    suite_fname = "%s.suite" % suite_name
    suite_path = os.sep.join([CIJ_ROOT, "suites", suite_fname])
    suite = open(suite_path).read().splitlines() if suite_name else []

    listing = tindex(search_path)
    if listing is None:
        return {}

    tests = suite if suite else listing
    if not tests:
        return {}

    cname = "Test"
    if suite:
        cname += suite_name.capitalize()

    cls = cls_factory(cname)
    for test in tests:              # Test method for each file
        tname = "test_%s" % test.replace(".sh", "_SHELL").replace(".py", "_PYTHON")
        setattr(cls, tname, cls.meta_test)
        classes[cname] = cls

    return classes

# Expose classes into scope for `unittest`, `py.test`, `nose`, `nose2`, or
# some other unittest framework to pick up the classes, instantiate
# them and run their test methods.
os.chdir(os.path.dirname(os.path.abspath(__file__)))

CLASSES = produce_classes(CIJ_TESTCASES, CIJ_SUITE)    # Create test classes
for CNAME in CLASSES:                               # Expose them in local scope
    CLS = CLASSES[CNAME]
    locals()[CNAME] = CLS
    del CLS                 # Avoid dual def. due to Python scope-leak
    del CNAME

if __name__ == "__main__":
    print("# Fix this")



"""
spdk.py  - Script providing operation of spdk
"""
import os
import re
import yaml
# from cij.struct.geometry import Geometry
# from cij.struct.identify import IdentifyNDS
# from cij.struct.chunk_info import DescriptorTable, get_sizeof_descriptor_table
import cij.util
import cij.nvme
import cij.ssh
import cij.bin
import cij


class Spdk(object):
    """spdk parameter"""
    def __init__(self):
        self.envs = cij.ENV.get("NVM_DEV_NAME")
        cmd = ["/opt/spdk/scripts/setup.sh"]
        STA, _, _ = cij.test.command(cmd)
        if STA != 0:
            raise RuntimeError("setup error: %r != expected: %r" % (STA, 0))
        cmd = ["nvm_dev info", self.envs]
        status, output, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("nvm_cmd idfy fail")

        result_list = output.split('\n')
        temp_list = []
        for i in range(0, len(result_list)):
            if result_list[i].startswith('#'):
                pass
            elif result_list[i].startswith('EAL'):
                pass
            elif result_list[i].startswith('['):
                pass
            elif result_list[i].startswith('Starting'):
                pass
            else:
                temp_list.append(result_list[i])
        strings = '\n'.join(temp_list)
        self.yml = yaml.load(strings)

    def get_env(self, key1, key2):
        return self.yml[key1][key2]

    @staticmethod
    def get_chunk_status(chk, yml):
        """Get item of chunk meta table"""
        cs = yml['rprt_descr'][chk]['cs']
        return cs

    def get_chunk_information(self, chk, lun, chunk_name):
        """Get chunk information"""
        cmd = ["nvm_cmd rprt_lun", self.envs,
               "%d %d > %s" % (chk, lun, chunk_name)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def is_bad_chunk(self, chk, yml):
        """Check the chunk is offline or not"""
        cs = self.get_chunk_status(chk, yml)
        if cs >= 8:
            return True
        return False

    def is_free_chunk(self, chk):
        """Check the chunk is free or not"""
        cs = self.get_chunk_status(chk)
        if cs & 0x1 != 0:
            return True
        return False

    def is_closed_chunk(self, chk):
        """Check the chunk is free or not"""
        cs = self.get_chunk_status(chk)
        if cs & 0x2 != 0:
            return True
        return False

    def is_open_chunk(self, chk):
        """Check the chunk is free or not"""
        cs = self.get_chunk_status(chk)
        if cs & 0x4 != 0:
            return True
        return False

    def s20_to_gen(self, pugrp, punit, chunk, sectr):
        """S20 unit to gen address"""
        cmd = ["nvm_addr s20_to_gen", self.envs,
               "%d %d %d %d" % (pugrp, punit, chunk, sectr)]
        status, stdout, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("cij.spdk.s20_to_gen: cmd fail")

        return int(re.findall(r"val: ([0-9a-fx]+)", stdout)[0], 16)

    def gen_to_dev(self, address):
        """Generic address to device address"""
        cmd = ["nvm_addr gen2dev", self.envs, "0x{:x}".format(address)]
        status, stdout, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("cij.spdk.gen_to_dev: cmd fail")

        return int(re.findall(r"dev: ([0-9a-fx]+)", stdout)[0], 16)

    def dev_to_gen(self, address):
        """Generic address to device address"""
        cmd = ["nvm_addr dev2gen", self.envs, "{:s}".format(address)]
        status, stdout, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("cij.spdk.dev_to_gen: cmd fail")
        return int(re.findall(r"gen: (0x\w+)", stdout)[0], 16)

    def vblk_erase(self, address):
        """nvm_vblk erase"""
        cmd = ["nvm_vblk erase", self.envs, "0x%x" % address]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vblk_write(self, address, file_name=None):
        """nvm_vblk write"""
        cmd = ["nvm_vblk write", self.envs, "0x%x" % address]
        if file_name:
            cmd += ["-i {}".format(file_name)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vblk_read(self, address, file_name=None):
        """nvm_vblk read"""
        cmd = ["nvm_vblk read", self.envs, "0x%x" % address]
        if file_name:
            cmd += ["-o {}".format(file_name)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vector_erase(self, address):
        """vector erase"""
        cmd = ["nvm_cmd erase", self.envs, "0x%x" % address]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vector_write(self, address_list, file_name):
        """nvm_cmd write"""
        address = ["0x{:x}".format(i) for i in address_list]
        cmd = ["nvm_cmd write", self.envs, " ".join(address), "-i {}".format(file_name)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vector_read(self, address_list, file_name):
        """nvm_cmd read"""
        address = ["0x{:x}".format(i) for i in address_list]
        cmd = ["nvm_cmd read", self.envs, " ".join(address),
               "-o {}".format(file_name)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def scalar_write(self, address, block_count, data_file, meta_file):
        """nvme write"""
        cmd = ["nvme", "write", self.envs, "-s 0x{:x}".format(address),
               "-c {}".format(block_count-1), "-d {}".format(data_file), "-M {}".format(meta_file),
               "-z 0x{:x}".format(block_count * self.get_env("dev_geo", "nbytes")),
               "-y 0x{:x}".format(block_count * self.get_env("dev_geo", "nbytes_oob"))]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def scalar_read(self, address, block_count, data_file, meta_file):
        """nvme read"""
        cmd = ["nvme", "read", self.envs, "-s 0x{:x}".format(address),
               "-c {}".format(block_count - 1), "-d {}".format(data_file),
               "-M {}".format(meta_file),
               "-z 0x{:x}".format(block_count * self.get_env("dev_geo", "nbytes")),
               "-y 0x{:x}".format(block_count * self.get_env("dev_geo", "nbytes_oob"))]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def test_scalar_ewr(self, seed=0x0, rmode=0x2, be_id=0x0):
        """nvm_test_cmd_ewr_scalar"""
        cmd = ["nvm_test_cmd_wre_scalar", self.envs, str(seed), str(rmode), str(be_id)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def test_vector_ewr(self, seed=0x0, rmode=0x2, be_id=0x0):
        """nvm_test_cmd_ewr_vector"""
        cmd = ["nvm_test_cmd_wre_vector", self.envs, str(seed), str(rmode), str(be_id)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def test_tlc_convert_slc(self, be_id=8):
        """ test tlc convert slc"""
        cmd = ["nvm_examples_slc-ex02-convert", self.envs, str(be_id)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def test_nvm_examples_slc_scan(self, be_id=8):
        """test nvm_examples_slc-ex01-scan"""
        cmd = ["nvm_examples_slc-ex01-scan", self.envs, str(be_id)]
        status, output, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("nvm_examples_slc-ex01-scan fail")
        return int(re.findall(r"nslc: ([0-9]+)", output)[0]), (re.findall(r"slba: (0x\w+)", output))

    def slc_erase(self, address, BE_ID=0x1, PMODE=0x0100):
        """slc erase"""
        cmd = ["NVM_CLI_BE_ID=0x%x" % BE_ID, "NVM_CLI_PMODE=0x%x" % PMODE, "nvm_cmd erase", self.envs, "0x%x" % address]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status


"""
geometry.py     - Device Geometry Data Structure
"""

from ctypes import Structure, c_uint8, c_uint16, c_uint32
# pylint: disable=locally-disabled, too-few-public-methods


class LBAF(Structure):
    """LBA format"""
    _pack_ = 1
    _fields_ = [
        ('GBL', c_uint8),   # 0 Group bit length
        ('PBL', c_uint8),   # PU bit length
        ('CBL', c_uint8),   # Chunk bit length
        ('BBL', c_uint8),   # Logical block bit length
        ('RSV0', c_uint32),
    ]


class Geometry(Structure):
    """Device Geometry Data Structure"""
    _pack_ = 1
    _fields_ = [
        ('MJR', c_uint8),           # Major Version Number
        ('MNR', c_uint8),           # Minor Version Number
        ('RSV0', c_uint8 * 6),      # Number of configuration groups
        ('LBAF', LBAF),             # LBA Format
        ('MCCAP', c_uint32),        # Media and Controller Capabilities
        ('RSV1', c_uint8 * 12),     # Current device operating mode
        ('WIT', c_uint8),           # Wear-level Index Delta Threshold
        ('RSV2', c_uint8 * 31),
        ('NUM_GRP', c_uint16),      # Number of Groups
        ('NUM_PU', c_uint16),       # Number of parallel units per group
        ('NUM_CHK', c_uint32),      # Number of chunks per parallel unit
        ('CLBA', c_uint32),         # Chunk Size
        ('RSV3', c_uint8 * 52),     # Description of the 4th group
        ('WS_MIN', c_uint32),       # Minimum Write Size
        ('WS_OPT', c_uint32),       # Optimal Write Size
        ('MW_CUNITS', c_uint32),    # Cache Minimum Write Size Units
        ('MAXOC', c_uint32),        # Maximum Open Chunks
        ('MAXOCPU', c_uint32),      # Maximum Open Chunks per PU
        ('RSV4', c_uint8 * 44),
        ('TRDT', c_uint32),         # tRD Typical
        ('TRDM', c_uint32),         # tRD Max
        ('TWRT', c_uint32),         # tWR Typical
        ('TWRM', c_uint32),         # tWR Max
        ('TCRST', c_uint32),        # tCRS Typical
        ('TCRSM', c_uint32),        # tCRS Max
        ('RSV5', c_uint8 * 40),
        ('RSV6', c_uint8 * 2816),
        ('VS', c_uint8 * 1024),     # Vendor Specific
    ]



#!/usr/bin/env python
#coding=utf-8
import os
import re
import commands
import cij.ssh
class libCfg(object):
    def __init__(self, fio):
        self.cfg_list = [['System Configuration','Information']]
        self.fio = fio
        
    def __getCpu(self):
        if not os.path.exists("/usr/sbin/dmidecode"):
            return
        
        status, output = cij.ssh.command("dmidecode -t 4")
        if status:
            print output
            print "[ERR] get CPU information failed!"
            return
        
        if "Version:" in output:
            cpu_name = re.findall("Version: ([^\n]+)", output)[0].strip()
            self.cfg_list.append(["CPU Mode Name", cpu_name])
        if "Current Speed:" in output:
            cpu_speed = re.findall("Current Speed: ([^\n]+)", output)[0].strip()
            self.cfg_list.append(["CPU Frequency", cpu_speed])
        if "Core Count:" in output:
            cpu_core = re.findall("Core Count: ([^\n]+)", output)[0].strip()
            self.cfg_list.append(["CPU Cores", "%sC" % cpu_core])
        if "Thread Count:" in output:
            cpu_thread = re.findall("Thread Count: ([^\n]+)", output)[0].strip()
            self.cfg_list.append(["CPU Threads", "%sT" % cpu_thread])

    def __getMem(self):
        if not os.path.exists("/usr/sbin/dmidecode"):
            return
        
        status, output = cij.ssh.command("dmidecode -t 17")
        if status:
            print output
            print "[ERR] get memory information failed!"
            return
        
        if "Type:" in output:
            type_list = re.findall("Type: ([^\n]+)", output)
            for mem_type in type_list:
                if "DDR" in mem_type:
                    mem_type = mem_type.strip()
                    break
            self.cfg_list.append(["Memory Type", mem_type])
        
        size_list = re.findall("Size: ([\d]+)", output)
        mem_uint = re.findall("Size: [\d]+ ([^\n]+)", output)[0].strip()
        mem_size = sum(map(int, size_list))
        self.cfg_list.append(["Memory Size", "%s %s" % (mem_size, mem_uint)])
    
    def __getPcie(self):
        if not os.path.exists("/usr/sbin/lspci"):
            return
        
        status, output = cij.ssh.command("lspci")
        if status:
            print output
            print "[ERR] get pcie information failed!"
            return
        
        if "Non-Volatile memory controller:" in output:
            pci_slot = re.findall("([^\n]+) Non-Volatile memory controller", output)[0].strip()
            self.cfg_list.append(["PCIe Slots", pci_slot])
    
    def __getRelease(self):
        if not os.path.exists("/usr/bin/lsb_release"):
            return
        
        status, output = cij.ssh.command("lsb_release -a")
        if status:
            print output
            print "[ERR] get linux release information failed!"
            return
        
        if "Description:" in output:
            release_info = re.findall("Description:([^\n]+)", output)[0].strip()
            self.cfg_list.append(["OS Release", release_info])
    
        status, output = cij.ssh.command("uname -r")
        if status:
            print output
            print "[ERR] get linux release information failed!"
            return
        self.cfg_list.append(["OS Kernel", output])
        
    def __getCnex(self):
        self.cfg_list.append(['Board Type',                  ''])
        self.cfg_list.append(['HW Accelerate Mode',          ''])
        self.cfg_list.append(['LBA Capacity',                ''])
        self.cfg_list.append(['NAND Type',                   ''])
        self.cfg_list.append(['NAND Capacity',               ''])
        self.cfg_list.append(['Over Provision Percentage',   ''])
        self.cfg_list.append(['PCIe Interface',              ''])
        self.cfg_list.append(['DDR type',                    ''])
        self.cfg_list.append(['NAND Clock',                  ''])
        self.cfg_list.append(['DDR clock',                   ''])
        self.cfg_list.append(['Core clock',                  ''])
        self.cfg_list.append(['Cache Hit',                   ''])
        self.cfg_list.append(['DDR Cache',                   ''])
        self.cfg_list.append(['FW Version',                  ''])
    
    def __getFio(self):
        if not os.path.exists(self.fio):
            return
        
        status, output = cij.ssh.command("%s --version" % self.fio)
        if status:
            print output
            print "[ERR] get fio information failed!"
            return
        
        self.cfg_list.append(["FIO Version", output])
    
    def __getBlankLine__(self):
        self.cfg_list.append([""])
    
    def getCfg(self):
        self.__getCpu()
        self.__getMem()
        self.__getPcie()
        
        self.__getBlankLine__()
        self.__getRelease()
        self.__getFio()
        
        self.__getBlankLine__()
        self.__getCnex()
        
        return self.cfg_list

class libTool(object):
    fio_2_1_10 = "../../tools/fio_2.1.10/fio"
    fio_2_19 = "../../tools/fio_2.19/fio"
    vdbench_504 = "../../tools/vdbench504/vdbench"

if __name__ == '__main__':
    cfg_list = libCfg("../../tools/fio_2.1.10/fio")
    for line in cfg_list.getCfg():
        print line
    


"""
    Environment for liblightnvm
"""
import cij.util
import cij.test
import cij.ssh
import cij

PREFIX = "NVM"
REQUIRED = ["DEV_NAME"]
EXPORTED = ["DEV_PATH"]

CHUNK_STATE_FREE = 0x1 << 0
CHUNK_STATE_CLOSED = 0x1 << 1
CHUNK_STATE_OPEN = 0x1 << 2
CHUNK_STATE_OFFLINE = 0x1 << 3
CHUNK_STATE_RSVD4 = 0x1 << 4
CHUNK_STATE_RSVD5 = 0x1 << 5
CHUNK_STATE_RSVD6 = 0x1 << 6
CHUNK_STATE_RSVD7 = 0x1 << 7


def env():
    """Verify NVME variables and construct exported variables"""

    if cij.ssh.env():
        cij.err("cij.nvm.env: invalid SSH environment")
        return 1

    nvm = cij.env_to_dict(PREFIX, REQUIRED)

    if "nvme" in nvm["DEV_NAME"]:
        nvm["DEV_PATH"] = "/dev/%s" % nvm["DEV_NAME"]
    else:
        nvm["DEV_PATH"] = "traddr:%s" % nvm["DEV_NAME"]

    cij.env_export(PREFIX, EXPORTED, nvm)

    return 0


def exists():
    """Verify that the ENV defined NVMe device exists"""

    if env():
        cij.err("cij.nvm.exists: Invalid NVMe ENV.")
        return 1

    nvm = cij.env_to_dict(PREFIX, EXPORTED + REQUIRED)

    cmd = ['[[ -b "%s" ]]' % nvm["DEV_PATH"]]
    rcode, _, _ = cij.ssh.command(cmd, shell=True, echo=False)

    return rcode


def dev_get_rprt(dev_name, pugrp=None, punit=None):
    """
    Get-log-page chunk information

    If the pugrp and punit is set, then provide report only for that pugrp/punit

    @returns the first chunk in the given state if one exists, None otherwise
    """

    cmd = ["nvm_cmd", "rprt_all", dev_name]
    if not (pugrp is None and punit is None):
        cmd = ["nvm_cmd", "rprt_lun", dev_name, str(pugrp), str(punit)]

    _, _, _, struct = cij.test.command_to_struct(cmd)
    if not struct:
        return None

    return struct["rprt_descr"]


def dev_get_chunk(dev_name, state, pugrp=None, punit=None):
    """
    Get a chunk-descriptor for the first chunk in the given state.

    If the pugrp and punit is set, then search only that pugrp/punit

    @returns the first chunk in the given state if one exists, None otherwise
    """

    rprt = dev_get_rprt(dev_name, pugrp, punit)
    if not rprt:
        return None

    return next((d for d in rprt if d["cs"] == state), None)


def addr_dev2gen(dev_name, addr):
    """Converts the given device address to gen-format"""

    return None



"""
liblight.py  - Script providing operation of liblightnvm for Spec 2.0
"""
import os
import re
from cij.struct.geometry import Geometry
from cij.struct.identify import IdentifyNDS
from cij.struct.chunk_info import get_descriptor_table, get_sizeof_descriptor_table
import cij.util
import cij.nvme
import cij.ssh
import cij.bin
import cij


class Nvm(object):
    """Operation of liblightnvm"""

    def __init__(self):
        if cij.ssh.env():
            raise RuntimeError("cij.liblight: invalid SSH environment")

        cij.nvme.env()
        nvme = cij.env_to_dict("NVME", ["DEV_NAME", "SPEC_VERSION"])

        self.envs = dict()
        self.envs["DEV_PATH"] = os.path.join("/dev", nvme["DEV_NAME"])

        self.envs["CHUNK_META_STRUCT"] = get_descriptor_table(nvme["SPEC_VERSION"])
        self.envs["CHUNK_META_SIZEOF"] = get_sizeof_descriptor_table(nvme["SPEC_VERSION"])

        cmd = ["nvme", "lnvm", "id-ns", self.envs["DEV_PATH"], "--raw-binary"]
        status, stdout, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("cij.liblight: lnvm id-ns fail")

        buff = cij.bin.Buffer(types=Geometry, length=1)
        buff.memcopy(stdout)
        geometry = buff[0]

        self.envs["MJR"] = geometry.MJR
        self.envs["NUM_GRP"] = geometry.NUM_GRP
        self.envs["NUM_PU"] = geometry.NUM_PU
        self.envs["NUM_CHK"] = geometry.NUM_CHK
        self.envs["CLBA"] = geometry.CLBA
        self.envs["CHUNKS"] = geometry.NUM_GRP * geometry.NUM_PU * geometry.NUM_CHK

        cmd = ["nvme", "id-ns", self.envs["DEV_PATH"], "--raw-binary"]
        status, stdout, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("cij.liblight: nvme id-ns fail")

        buff = cij.bin.Buffer(types=IdentifyNDS, length=1)
        buff.memcopy(stdout)
        identify = buff[0]

        self.envs["NBYTES"] = 2 ** identify.LBAF[identify.FLBAS].LBADS
        self.envs["NBYTES_OOB"] = identify.LBAF[identify.FLBAS].MS
        self.envs["FLBAS"] = identify.FLBAS

    def get_envs(self, key):
        """Get environment of liblightnvm"""
        return self.envs[key]

    def get_chunk_meta(self, meta_file):
        """Get chunk meta table"""
        chunks = self.envs["CHUNKS"]
        if cij.nvme.get_meta(0, chunks * self.envs["CHUNK_META_SIZEOF"], meta_file):
            raise RuntimeError("cij.liblight.get_chunk_meta: fail")

        chunk_meta = cij.bin.Buffer(types=self.envs["CHUNK_META_STRUCT"], length=chunks)
        chunk_meta.read(meta_file)
        return chunk_meta

    def get_chunk_meta_item(self, chunk_meta, grp, pug, chk):
        """Get item of chunk meta table"""
        num_chk = self.envs["NUM_CHK"]
        num_pu = self.envs["NUM_PU"]
        index = grp * num_pu * num_chk + pug * num_chk + chk
        return chunk_meta[index]

    def is_free_chunk(self, chunk_meta, grp, pug, chk):
        """Check the chunk is free or not"""
        meta = self.get_chunk_meta_item(chunk_meta, grp, pug, chk)
        if meta.CS & 0x1 != 0:
            return True
        return False

    def is_closed_chunk(self, chunk_meta, grp, pug, chk):
        """Check the chunk is free or not"""
        meta = self.get_chunk_meta_item(chunk_meta, grp, pug, chk)
        if meta.CS & 0x2 != 0:
            return True
        return False

    def is_open_chunk(self, chunk_meta, grp, pug, chk):
        """Check the chunk is free or not"""
        meta = self.get_chunk_meta_item(chunk_meta, grp, pug, chk)
        if meta.CS & 0x4 != 0:
            return True
        return False

    def is_bad_chunk(self, chunk_meta, grp, pug, chk):
        """Check the chunk is offline or not"""
        meta = self.get_chunk_meta_item(chunk_meta, grp, pug, chk)
        if meta.CS & 0x8 != 0:
            return True
        return False

    def s20_to_gen(self, pugrp, punit, chunk, sectr):
        """S20 unit to generic address"""
        cmd = ["nvm_addr s20_to_gen", self.envs["DEV_PATH"],
               "%d %d %d %d" % (pugrp, punit, chunk, sectr)]
        status, stdout, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("cij.liblight.s20_to_gen: cmd fail")

        return int(re.findall(r"val: ([0-9a-fx]+)", stdout)[0], 16)

    def gen_to_dev(self, address):
        """Generic address to device address"""
        cmd = ["nvm_addr gen2dev", self.envs["DEV_PATH"], "0x{:x}".format(address)]
        status, stdout, _ = cij.ssh.command(cmd, shell=True)
        if status:
            raise RuntimeError("cij.liblight.gen_to_dev: cmd fail")

        return int(re.findall(r"dev: ([0-9a-fx]+)", stdout)[0], 16)

    def vblk_erase(self, address):
        """nvm_vblk erase"""
        cmd = ["nvm_vblk erase", self.envs["DEV_PATH"], "0x%x" % address]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vblk_write(self, address, meta=False):
        """nvm_vblk write"""
        cmd = list()
        if meta:
            cmd.append("NVM_CLI_META_MODE=1")
        cmd += ["nvm_vblk write", self.envs["DEV_PATH"], "0x%x" % address]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vblk_read(self, address, meta=False):
        """nvm_vblk read"""
        cmd = list()
        if meta:
            cmd.append("NVM_CLI_META_PR=1")
        cmd += ["nvm_vblk read", self.envs["DEV_PATH"], "0x%x" % address]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vector_erase(self, address):
        """nvm_cmd erase"""
        cmd = ["nvm_cmd erase", self.envs["DEV_PATH"], "0x{:x}".format(address)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vector_write(self, address_list, file_name=None):
        """nvm_cmd write"""
        address = ["0x{:x}".format(i) for i in address_list]
        cmd = ["nvm_cmd write", self.envs["DEV_PATH"], " ".join(address)]
        if file_name:
            cmd += ["-i {}".format(file_name)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def vector_read(self, address_list, file_name=None):
        """nvm_cmd read"""
        address = ["0x{:x}".format(i) for i in address_list]
        cmd = ["nvm_cmd read", self.envs["DEV_PATH"], " ".join(address)]
        if file_name:
            cmd += ["-o {}".format(file_name)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def scalar_write(self, address, block_count, data_file, meta_file):
        """nvme write"""
        cmd = ["nvme", "write", self.envs["DEV_PATH"], "-s 0x{:x}".format(address),
               "-c {}".format(block_count-1), "-d {}".format(data_file), "-M {}".format(meta_file),
               "-z 0x{:x}".format(block_count * self.envs["NBYTES"]),
               "-y 0x{:x}".format(block_count * self.envs["NBYTES_OOB"])]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def scalar_read(self, address, block_count, data_file, meta_file):
        """nvme read"""
        cmd = ["nvme", "read", self.envs["DEV_PATH"], "-s 0x{:x}".format(address),
               "-c {}".format(block_count - 1), "-d {}".format(data_file),
               "-M {}".format(meta_file),
               "-z 0x{:x}".format(block_count * self.envs["NBYTES"]),
               "-y 0x{:x}".format(block_count * self.envs["NBYTES_OOB"])]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def test_scalar_ewr(self, seed=0x0, rmode=0x2, be_id=0x1):
        """nvm_test_cmd_ewr_scalar"""
        cmd = ["nvm_test_cmd_ewr_scalar", self.envs["DEV_PATH"], str(seed), str(rmode), str(be_id)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status

    def test_vector_ewr(self, seed=0x0, rmode=0x2, be_id=0x1):
        """nvm_test_cmd_ewr_vector"""
        cmd = ["nvm_test_cmd_ewr_vector", self.envs["DEV_PATH"], str(seed), str(rmode), str(be_id)]
        status, _, _ = cij.ssh.command(cmd, shell=True)
        return status



"""
nvme.py    - Script providing operation of NVMe

Functions:
    nvme.env()          - Prepare environment for NVMe
    nvme.fmt()          - Format NVMe device
    nvme.exists()       - Check NVMe device is exists or not
    nvme.get_meta()     - Get chunk meta to file
    nvme.comp_meta()    - Compare chunk meta

Require:
    NVME_DEV_NAME       - NVMe device name
"""
import os
import traceback
from cij.struct.identify import IdentifyCDS
from cij.struct.chunk_info import get_descriptor_table, get_sizeof_descriptor_table
import cij.util
import cij.ssh
import cij.bin
import cij

PREFIX = "NVME"
REQUIRED = ["DEV_NAME"]
EXPORTED = ["DEV_PATH", "LNVM_VERSION", "SPEC_VERSION", "LNVM_NUM_CHUNKS", "LNVM_NUM_LUNS",
            "LNVM_NUM_CHS", "LNVM_TOTAL_LUNS", "LNVM_TOTAL_CHUNKS", "LNVM_CHUNK_META_SIZE",
            "LNVM_CHUNK_META_LENGTH"]


def cat_file(path):
    """Cat file and return content"""

    cmd = ["cat", path]
    status, stdout, _ = cij.ssh.command(cmd, shell=True, echo=True)
    if status:
        raise RuntimeError("cij.nvme.env: cat %s failed" % path)
    return stdout.strip()


def env():
    """Verify NVME variables and construct exported variables"""

    if cij.ssh.env():
        cij.err("cij.nvme.env: invalid SSH environment")
        return 1

    nvme = cij.env_to_dict(PREFIX, REQUIRED)

    nvme["DEV_PATH"] = os.path.join("/dev", nvme["DEV_NAME"])

    # get version, chunks, luns and chs
    try:
        sysfs = os.path.join("/sys/class/block", nvme["DEV_NAME"], "lightnvm")

        nvme["LNVM_VERSION"] = cat_file(os.path.join(sysfs, "version"))
        if nvme["LNVM_VERSION"] == "2.0":
            luns = "punits"
            chs = "groups"
        elif nvme["LNVM_VERSION"] == "1.2":
            luns = "num_luns"
            chs = "num_channels"
        else:
            raise RuntimeError("cij.nvme.env: invalid lnvm version: %s" % nvme["LNVM_VERSION"])

        nvme["LNVM_NUM_CHUNKS"] = cat_file(os.path.join(sysfs, "chunks"))
        nvme["LNVM_NUM_LUNS"] = cat_file(os.path.join(sysfs, luns))
        nvme["LNVM_NUM_CHS"] = cat_file(os.path.join(sysfs, chs))

        nvme["LNVM_TOTAL_LUNS"] = str(int(nvme["LNVM_NUM_LUNS"]) * int(nvme["LNVM_NUM_CHS"]))
        nvme["LNVM_TOTAL_CHUNKS"] = str(int(nvme["LNVM_TOTAL_LUNS"]) * int(nvme["LNVM_NUM_CHUNKS"]))

        # get spec version by identify namespace data struct
        if nvme["LNVM_VERSION"] == "2.0":
            cmd = ["nvme", "id-ctrl", nvme["DEV_PATH"], "--raw-binary"]
            status, stdout, _ = cij.ssh.command(cmd, shell=True)
            if status:
                raise RuntimeError("cij.nvme.env: nvme id-ctrl fail")

            buff = cij.bin.Buffer(types=IdentifyCDS, length=1)
            buff.memcopy(stdout)

            if buff[0].VS[1023] == 0x5a:
                nvme["SPEC_VERSION"] = "Denali"
            else:
                nvme["SPEC_VERSION"] = "Spec20"
        else:
            nvme["SPEC_VERSION"] = "Spec12"

        # get chunk meta information
        nvme["LNVM_CHUNK_META_LENGTH"] = str(get_sizeof_descriptor_table(nvme["SPEC_VERSION"]))
        nvme["LNVM_CHUNK_META_SIZE"] = str(int(nvme["LNVM_CHUNK_META_LENGTH"]) *
                                           int(nvme["LNVM_TOTAL_CHUNKS"]))

    except StandardError:
        traceback.print_exc()
        return 1

    cij.env_export(PREFIX, EXPORTED, nvme)

    return 0


def fmt(lbaf=3):
    """Do format for NVMe device"""

    if env():
        cij.err("cij.nvme.exists: Invalid NVMe ENV.")
        return 1

    nvme = cij.env_to_dict(PREFIX, EXPORTED + REQUIRED)

    cmd = ["nvme", "format", nvme["DEV_PATH"], "-l", str(lbaf)]
    rcode, _, _ = cij.ssh.command(cmd, shell=True)

    return rcode


def exists():
    """Verify that the ENV defined NVMe device exists"""

    if env():
        cij.err("cij.nvme.exists: Invalid NVMe ENV.")
        return 1

    nvme = cij.env_to_dict(PREFIX, EXPORTED + REQUIRED)

    cmd = ["[[", "-b", nvme["DEV_PATH"], "]]"]
    rcode, _, _ = cij.ssh.command(cmd, shell=True, echo=False)
    if rcode:
        return False

    return True


def get_meta(offset, length, output):
    """Get chunk meta of NVMe device"""

    if env():
        cij.err("cij.nvme.meta: Invalid NVMe ENV.")
        return 1

    nvme = cij.env_to_dict(PREFIX, EXPORTED + REQUIRED)

    max_size = 0x40000
    with open(output, "wb") as fout:
        for off in range(offset, length, max_size):
            size = min(length - off, max_size)
            cmd = ["nvme get-log",
                   nvme["DEV_PATH"],
                   "-i 0xca",
                   "-o 0x%x" % off,
                   "-l 0x%x" % size,
                   "-b"]
            status, stdout, _ = cij.ssh.command(cmd, shell=True)
            if status:
                cij.err("cij.nvme.meta: Error get chunk meta")
                return 1

            fout.write(stdout)

    return 0


def comp_meta(file_bef, file_aft, mode="pfail"):
    """Compare chunk meta, mode=[pfail, power, reboot]"""
    if env():
        cij.err("cij.nvme.comp_meta: Invalid NVMe ENV.")
        return 1

    nvme = cij.env_to_dict(PREFIX, EXPORTED + REQUIRED)
    num_chk = int(nvme["LNVM_TOTAL_CHUNKS"])

    meta_bef = cij.bin.Buffer(types=get_descriptor_table(nvme['SPEC_VERSION']), length=num_chk)
    meta_aft = cij.bin.Buffer(types=get_descriptor_table(nvme['SPEC_VERSION']), length=num_chk)
    meta_bef.read(file_bef)
    meta_aft.read(file_aft)

    for chk in range(num_chk):
        ignore = ["WL", "RSV0"]

        # PFAIL: BEFORE IS OPEN CHUNK, WRITE POINTER IS NOT SURE, IGNORE
        if mode == "pfail" and meta_bef[chk].CS == 4:
            ignore.append("WP")

        # COMPARE CHUNK META
        if meta_bef.compare(meta_aft, chk, ignore=ignore):
            cij.warn("META_BUFF_BEF[%s]:" % chk)
            meta_bef.dump(chk)
            cij.warn("META_BUFF_AFT[%s]:" % chk)
            meta_aft.dump(chk)
            cij.err("Error compare, CHUNK: %s" % chk)
            return 1

    return 0



#!/usr/bin/env python
"""
identify.py     - NVMe Identify
"""
from ctypes import Structure, c_uint8, c_uint16, c_uint32, c_uint64
# pylint: disable=locally-disabled, too-few-public-methods


class IdentifyCDS(Structure):
    """Identify - Identify Controller Data Structure"""
    _pack_ = 1
    _fields_ = [
        ('VID', c_uint16),          # PCI Vendor ID
        ('SSVID', c_uint16),        # PCI Subsystem Vendor ID
        ('SN', c_uint8 * 20),       # Serial Number
        ('MN', c_uint8 * 40),       # Model Number
        ('FR', c_uint64),           # Firmware Revision
        ('RAB', c_uint8),           # Recommended Arbitration Burst
        ('IEEE', c_uint32, 24),     # IEEE OUI Identifier
        ('CMIC', c_uint32, 8),      # Controller Multi-Path I/O and Namespace Sharing Capabilities
        ('MDTS', c_uint8),          # Maximum Data Transfer Size
        ('CNTLID', c_uint16),       # Controller ID
        ('VER', c_uint32),          # Version
        ('RTD3R', c_uint32),        # RTD3 Resume Latency
        ('RTD3E', c_uint32),        # RTD3 Entry Latency
        ('OAES', c_uint32),         # Optional Asynchronous Events Supported
        ('RSV0', c_uint8 * 144),
        ('NMIS', c_uint8 * 16),     # Refer to the NVMe Management Interface Specification
        ('OACS', c_uint16),         # Optional Admin Command Support
        ('ACL', c_uint8),           # Abort Command Limit
        ('AERL', c_uint8),          # Asynchronous Event Request Limit
        ('FRMW', c_uint8),          # Firmware Updates
        ('LPA', c_uint8),           # Log Page Attributes
        ('ELPE', c_uint8),          # Error Log Page Entries
        ('NPSS', c_uint8),          # Number of Power States Support
        ('AVSCC', c_uint8),         # Admin Vendor Specific Command Configuration
        ('APSTA', c_uint8),         # Autonomous Power State Transition Attributes
        ('WCTEMP', c_uint16),       # Warning Composite Temperature Threshold
        ('CCTEMP', c_uint16),       # Critical Composite Temperature Threshold
        ('MTFA', c_uint16),         # Maximum Time for Firmware Activation
        ('HMPRE', c_uint32),        # Host Memory Buffer Preferred Size
        ('HMMIN', c_uint32),        # Host Memory Buffer Minimum Size
        ('TNVMCAP', c_uint8 * 16),  # Total NVM Capacity
        ('UNVMCAP', c_uint8 * 16),  # Unallocated NVM Capacity
        ('RPMBS', c_uint32),        # Replay Protected Memory Block Support
        ('RSV1', c_uint8 * 196),
        ('SQES', c_uint8),          # Submission Queue Entry Size
        ('CQES', c_uint8),          # Completion Queue Entry Size
        ('RSV2', c_uint8 * 2),
        ('NN', c_uint32),           # Number of Namespaces
        ('ONCS', c_uint16),         # Optional NVM Command Support
        ('FUSES', c_uint16),        # Fused Operation Support
        ('FNA', c_uint8),           # Format NVM Attributes
        ('VWC', c_uint8),           # Volatile Write Cache
        ('AWUN', c_uint16),         # Atomic Write Unit Normal
        ('AWUPF', c_uint16),        # Atomic Write Unit Power Fail
        ('NVSCC', c_uint8),         # NVM Vendor Specific Command Configuration
        ('RSV3', c_uint8),
        ('ACWU', c_uint16),         # Atomic Compare & Write Unit
        ('RSV4', c_uint8 * 2),
        ('SGLS', c_uint32),         # SGL Support
        ('RSV5', c_uint8 * 164),
        ('RSV6', c_uint8 * 1344),
        ('PSD0', c_uint8 * 32),     # Power State 0 Descriptor
        ('PSD1', c_uint8 * 32),     # Power State 1 Descriptor
        ('PSD2', c_uint8 * 32),     # Power State 2 Descriptor
        ('PSD3', c_uint8 * 32),     # Power State 3 Descriptor
        ('PSD4', c_uint8 * 32),     # Power State 4 Descriptor
        ('PSD5', c_uint8 * 32),     # Power State 5 Descriptor
        ('PSD6', c_uint8 * 32),     # Power State 6 Descriptor
        ('PSD7', c_uint8 * 32),     # Power State 7 Descriptor
        ('PSD8', c_uint8 * 32),     # Power State 8 Descriptor
        ('PSD9', c_uint8 * 32),     # Power State 9 Descriptor
        ('PSD10', c_uint8 * 32),    # Power State 10 Descriptor
        ('PSD11', c_uint8 * 32),    # Power State 11 Descriptor
        ('PSD12', c_uint8 * 32),    # Power State 12 Descriptor
        ('PSD13', c_uint8 * 32),    # Power State 13 Descriptor
        ('PSD14', c_uint8 * 32),    # Power State 14 Descriptor
        ('PSD15', c_uint8 * 32),    # Power State 15 Descriptor
        ('PSD16', c_uint8 * 32),    # Power State 16 Descriptor
        ('PSD17', c_uint8 * 32),    # Power State 17 Descriptor
        ('PSD18', c_uint8 * 32),    # Power State 18 Descriptor
        ('PSD19', c_uint8 * 32),    # Power State 19 Descriptor
        ('PSD20', c_uint8 * 32),    # Power State 20 Descriptor
        ('PSD21', c_uint8 * 32),    # Power State 21 Descriptor
        ('PSD22', c_uint8 * 32),    # Power State 22 Descriptor
        ('PSD23', c_uint8 * 32),    # Power State 23 Descriptor
        ('PSD24', c_uint8 * 32),    # Power State 24 Descriptor
        ('PSD25', c_uint8 * 32),    # Power State 25 Descriptor
        ('PSD26', c_uint8 * 32),    # Power State 26 Descriptor
        ('PSD27', c_uint8 * 32),    # Power State 27 Descriptor
        ('PSD28', c_uint8 * 32),    # Power State 28 Descriptor
        ('PSD29', c_uint8 * 32),    # Power State 29 Descriptor
        ('PSD30', c_uint8 * 32),    # Power State 30 Descriptor
        ('PSD31', c_uint8 * 32),    # Power State 31 Descriptor
        ('VS', c_uint8 * 1024),     # Vendor Specific
    ]


class IdentifyPSDDS(Structure):
    """Identify - Power State Descriptor Data Structure"""
    _pack_ = 1
    _fields_ = [
        ('MP', c_uint16),           # Active Power Scale
        ('RSV0', c_uint8),
        ('MPS', c_uint8, 1),        # Max Power Scale
        ('NOPS', c_uint8, 1),       # Non-Operational State
        ('RSV1', c_uint8, 6),
        ('ENLAT', c_uint32),        # Entry Latency
        ('EXLAT', c_uint32),        # Exit Latency
        ('RRT', c_uint8, 5),        # Relative Read Throughput
        ('RSV2', c_uint8, 3),
        ('RRL', c_uint8, 5),        # Relative Read Latency
        ('RSV3', c_uint8, 3),
        ('RWT', c_uint8, 5),        # Relative Write Throughput
        ('RSV4', c_uint8, 3),
        ('RWL', c_uint8, 5),        # Relative Write Latency
        ('RSV5', c_uint8, 3),
        ('IDLP', c_uint16),         # Idle Power
        ('RSV6', c_uint8, 6),
        ('IPS', c_uint8, 2),        # Idle Power Scale
        ('RSV7', c_uint8),
        ('ACTP', c_uint16),         # Active Power
        ('APW', c_uint8, 3),        # Active Power Workload
        ('RSV8', c_uint8, 3),
        ('APS', c_uint8, 2),        # Active Power Scale
        ('RSV9', c_uint8 * 9),
    ]


class IdentifyLbaf(Structure):
    """Identify - LBA Format Data Structure, NVM Command Set Specific"""
    _pack_ = 1
    _fields_ = [
        ('MS', c_uint16),       # Metadata Size
        ('LBADS', c_uint8),     # LBA Data Size
        ('RP', c_uint8, 2),     # Relative Performance
        ('RSV', c_uint8, 6),
    ]


class IdentifyNDS(Structure):
    """Identify - Identify Namespace Data Structure, NVM Command Set Specific"""
    _pack_ = 1
    _fields_ = [
        ('NSZE', c_uint64),         # Namespace Size
        ('NCAP', c_uint64),         # Namespace Capacity
        ('NUSE', c_uint64),         # Namespace Utilization
        ('NSFEAT', c_uint8),        # Namespace Features
        ('NLBAF', c_uint8),         # Number of LBA Formats
        ('FLBAS', c_uint8),         # Formatted LBA Size
        ('MC', c_uint8),            # Metadata Capabilities
        ('DPC', c_uint8),           # End-to-end Data Protection Capabilities
        ('DPS', c_uint8),           # End-to-end Data Protection Type Settings
        ('NMIC', c_uint8),          # Namespace Multi-path I/O and Namespace Sharing Capabilities
        ('RESCAP', c_uint8),        # Reservation Capabilities
        ('FPI', c_uint8),           # Format Progress Indicator
        ('RSV0', c_uint8),
        ('NAWUN', c_uint16),        # Namespace Atomic Write Unit Normal
        ('NAWUPF', c_uint16),       # Namespace Atomic Write Unit Power Fail
        ('NACWU', c_uint16),        # Namespace Atomic Compare & Write Unit
        ('NABSN', c_uint16),        # Namespace Atomic Boundary Size Normal
        ('NABO', c_uint16),         # Namespace Atomic Boundary Offset
        ('NABSPF', c_uint16),       # Namespace Atomic Boundary Size Power Fail
        ('RSV1', c_uint8 * 2),
        ('NVMECAP', c_uint8 * 16),  # NVM Capacity
        ('RSV2', c_uint8 * 40),
        ('NGUID', c_uint8 * 16),    # Namespace Globally Unique Identifier
        ('EUI64', c_uint64),        # IEEE Extended Unique Identifier
        ('LBAF', IdentifyLbaf * 16),        # LBA Format Support
        ('RSV3', c_uint8 * 192),
        ('VS', c_uint8 * 3712),     # Vendor Specific
    ]


class IdentifyLFDS(Structure):
    """Identify - LBA Format Data Structure, NVM Command Set Specific"""
    _pack_ = 1
    _fields_ = [
        ('MS', c_uint32, 16),       # Metadata Size
        ('LBADS', c_uint32, 8),     # LBA Data Size
        ('RP', c_uint32, 2),        # Relative Performance
        ('RSV', c_uint32, 6),
    ]



"""
chunk_info.py     - Get Log Page, Chunk Information
"""

from ctypes import Structure, c_uint8, c_uint64, sizeof
# pylint: disable=locally-disabled, too-few-public-methods


class DescriptorTableSpec20(Structure):
    """Chunk Descriptor Table"""
    _pack_ = 1
    _fields_ = [
        ('CS', c_uint8),        # Chunk State
        ('CT', c_uint8),        # Chunk Type
        ('WL', c_uint8),        # Wear-level Index
        ('RSV0', c_uint8 * 5),
        ('SLBA', c_uint64),     # Starting LBA
        ('CNLB', c_uint64),     # Number of blocks in chunk
        ('WP', c_uint64),       # Write Pointer
    ]


class DescriptorTableDenali(Structure):
    """Chunk Descriptor Table"""
    _pack_ = 1
    _fields_ = [
        ('CS', c_uint8),        # Chunk State
        ('CT', c_uint8),        # Chunk Type
        ('WL', c_uint8),        # Wear-level Index
        ('RSV0', c_uint8 * 5),
        ('SLBA', c_uint64),     # Starting LBA
        ('CNLB', c_uint64),     # Number of blocks in chunk
        ('WP', c_uint64),       # Write Pointer
        ('ILBA', c_uint64),     # Number of invalid LBAs
        ('RSV1', c_uint64 * 3),
    ]


def get_descriptor_table(version="Denali"):
    """
    Get descriptor table by version(denali, spec20)
    """
    if version == "Denali":
        return DescriptorTableDenali
    elif version == "Spec20":
        return DescriptorTableSpec20
    elif version == "Spec12":
        return None
    else:
        raise RuntimeError("Error version!")


def get_sizeof_descriptor_table(version="Denali"):
    """
    Get sizeof DescriptorTable
    """
    if version == "Denali":
        return sizeof(DescriptorTableDenali)
    elif version == "Spec20":
        return sizeof(DescriptorTableSpec20)
    elif version == "Spec12":
        return 0
    else:
        raise RuntimeError("Error version!")



#
# ipmi.py - Script providing convenience functions for invoking ipmi
#
# Functions:
#
# ipmi.env - Sets default vars for ipmi wrapping
# ipmi.cmd <CMD> - Execute ipmitool command <CMD>
# ipmi.on    - Power on system
# ipmi.off - Power off system
# ipmi.reset - Power reset system
#
# Variables:
#
# IPMI_USER - login on server
# IPMI_PASS - password to server
# IPMI_HOST - server host
# IPMI_PORT - server port
#
import cij.util
import cij.ssh
import cij

PREFIX = "IMPI"
REQUIRED = ["USER", "PASS", "HOST", "PORT"]
EXPORTED = []


def env():
    """Verify IPMI environment"""

    ipmi = cij.env_to_dict(PREFIX, REQUIRED)

    if ipmi is None:
        ipmi["USER"] = "admin"
        ipmi["PASS"] = "admin"
        ipmi["HOST"] = "localhost"
        ipmi["PORT"] = "623"
        cij.info("ipmi.env: USER: %s, PASS: %s, HOST: %s, PORT: %s" % (
            ipmi["USER"], ipmi["PASS"], ipmi["HOST"], ipmi["PORT"]
        ))

    cij.env_export(PREFIX, EXPORTED, ipmi)

    return 0


def command(cmd):
    """Send command to ipmi"""

    env()

    ipmi = cij.env_to_dict(PREFIX, EXPORTED + REQUIRED)

    cmd = "ipmitool -U %s -P %s -H %s -p %s %s" % (
        ipmi["USER"], ipmi["PASS"], ipmi["HOST"], ipmi["PORT"], cmd)
    cij.info("ipmi.command: %s" % cmd)

    return cij.util.execute(cmd, shell=True, echo=True)


def on():
    """Target On"""

    command("power on")


def off():
    """Target Off"""

    command("power off")


def reset():
    """Target reset"""

    command("power reset")
